package com.iot.modbus_rtcp.netty;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

@Slf4j
public class ChannelGroup {

    private final AtomicLong lastCleanupChannel = new AtomicLong(0);
    private final ConcurrentHashMap<String, SyncPriorityChannel> mChannelMap = new ConcurrentHashMap<>(512);

    public void put(String identity, SyncPriorityChannel channel) {
        this.mChannelMap.put(identity, channel);
    }

    public SyncPriorityChannel get(String identity) {
        return this.mChannelMap.get(identity);
    }

    public SyncPriorityChannel get(String ip, int port) {
        return this.mChannelMap.get(getKey(ip, port));
    }

    public SyncPriorityChannel getAssociatedChannel(String identity, String gatewayHeartbeat, String ip, int port) {
        SyncPriorityChannel channel = this.get(gatewayHeartbeat);
        if (Objects.isNull(channel)) {
            List<String> sameIPKeys = this.mChannelMap.keySet().stream()
                    .filter(key -> StringUtils.contains(key, ":"))
                    .filter(key -> StringUtils.startsWith(key, ip + ":"))
                    .toList();
            if (sameIPKeys.size() == 1) {
                channel = this.get(sameIPKeys.getFirst());
            } else {
                Optional<String> keyOptional = sameIPKeys.stream().filter(key -> StringUtils.endsWith(key, ":" + port)).findFirst();
                if (keyOptional.isPresent()) {
                    channel = this.get(keyOptional.get());
                }
            }
            if (Objects.isNull(channel)) {
                if (this.lastCleanupChannel.get() == 0 || TimeUnit.MILLISECONDS.toDays(System.currentTimeMillis() - this.lastCleanupChannel.get()) >= 1) {
                    log.error("存在心跳包{}[{}]对应{}:{}关系映射错误问题，距离上次清理链接已操作24小时，尝试清理链接修复", identity, gatewayHeartbeat, ip, port);
                    new HashSet<>(this.mChannelMap.keySet())
                            .stream()
                            .filter(key -> StringUtils.contains(key, ":"))
                            .forEach(key -> Optional.ofNullable(this.mChannelMap.remove(key)).ifPresent(SyncPriorityChannel::close));
                } else {
                    log.error("存在心跳包{}[{}]对应{}:{}关系映射错误问题", identity, gatewayHeartbeat, ip, port);
                }
            }
        }
        return channel;
    }

    public void remove(String ip, int port) {
        this.mChannelMap.remove(getKey(ip, port));
    }

    public void remove(String identity) {
        this.mChannelMap.remove(identity);
    }

    public Set<String> onlineGateway() {
        return this.mChannelMap.keySet()
                .stream()
                .filter(identifier -> !StringUtils.contains(identifier, ":"))
                .collect(Collectors.toSet());
    }

    public Long onlineTotal() {
        return this.mChannelMap.keySet()
                .stream()
                .filter(identifier -> !StringUtils.contains(identifier, ":"))
                .count();
    }

    public void see() {
        log.info("当前连接有:{}", new ArrayList<>(this.mChannelMap.keySet()));
    }

    public static String getKey(String ip, int port) {
        return ip + ":" + port;
    }
}
