package com.iot.modbus_rtcp.netty;

//import com.iot.modbus.rtcp.commons.keymgr.LicenseValidator;

import com.iot.modbus.rtcp.commons.keymgr.LicenseValidator;
import com.iot.modbus_rtcp.utils.SpringUtil;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class NettyServer extends ChannelInitializer<SocketChannel> {
    private EventLoopGroup bossGroup;
    private EventLoopGroup workerGroup;
    private ChannelFuture future;
    private int port, nThread;

    @Getter
    private ChannelGroup group;
    private ModbusSender sender;
    private ChannelHandler handler;

    /**
     * 创建指定服务端口，指定线程数的服务端
     *
     * @param port    服务端口
     * @param nThread 执行线程池线程数
     */
    public NettyServer(int port, int nThread) {
        this.port = port;
        this.nThread = nThread;
        this.group = new ChannelGroup();
        this.sender = new ModbusSender(this.group);
        this.handler = new SyncHandler(this.group);
    }

    /**
     * 启动服务
     */
    public void start() {
        this.bossGroup = new NioEventLoopGroup(1);
        this.workerGroup = new NioEventLoopGroup(this.nThread);
        ServerBootstrap bootstrap = new ServerBootstrap();
        bootstrap.group(this.bossGroup, this.workerGroup)
                .channel(NioServerSocketChannel.class)
                .handler(new LoggingHandler(LogLevel.INFO))
                .childOption(ChannelOption.SO_KEEPALIVE, true)
                .childHandler(this);
        this.future = bootstrap.bind(this.port);
    }

    /**
     * 停止服务
     */
    public void stop() {
        this.future.channel().closeFuture();
        this.workerGroup.shutdownGracefully();
        this.bossGroup.shutdownGracefully();
    }

    public ModbusSender sender() {
        return this.sender;
    }

    @Override
    protected void initChannel(SocketChannel ch) throws Exception {
        ChannelPipeline pipeline = ch.pipeline();

        log.info("创建channel:{}", ch);

        String ip = ch.remoteAddress().getHostString();
        //过滤掉docker 网关请求
        if ("172.17.0.1".equals(ip)) return;
        SpringUtil.getBean(LicenseValidator.class).verifyNumber(ch::close);
        int port = ch.remoteAddress().getPort();
        this.group.put(ChannelGroup.getKey(ip, port), new SyncPriorityChannel(ch));

        pipeline.addLast(new ModbusEncoder());
        pipeline.addLast("decoder", new ModbusDecoder(this.group));
//        pipeline.addLast(new ReadTimeoutHandler(10000, TimeUnit.MILLISECONDS));
        pipeline.addLast(this.handler);
    }
}
